﻿using UnityEngine;
using System;
using PixelCrushers.DialogueSystem;

/// <summary>
/// When you attach this script to an actor, conversations involving that actor will be
/// logged to the console and to a file.
/// </summary>

[AddComponentMenu("Dialogue System/Miscellaneous/Conversation Logger")]
public class ARConversationLogger : MonoBehaviour
{

    public Color playerColor = Color.blue;
    public Color npcColor = Color.red;

    public string fileArchiveName;
    GameObject fileArchiveGameObject;
    FileArchive archive;
    bool systemOk;

    void Start() {
        SetupArchive();
    }

    void SetupArchive()
    {
        systemOk = false;
        try
        {
            fileArchiveGameObject = GameObject.Find(fileArchiveName);
            archive = fileArchiveGameObject.GetComponent<FileArchive>();
            systemOk = true;
        }
        catch (Exception e)
        {
            Debug.Log("Error in " + this.gameObject + "file archive name has not been set");
            Debug.Log(e);
            systemOk = false;
        }
        finally { }
    }

    void EchoMessage(string s)
    {
        Debug.Log(s);
        if (systemOk) archive.AddTextToFile(s);
    }

    public void OnConversationStart(Transform actor)
    {
        string temp = (string.Format("{0}: Starting conversation with {1}", name, GetActorName(actor)));
        EchoMessage(temp);

    }

    public void OnConversationLine(Subtitle subtitle)
    {
        //Debug.Log(string.Format("<color={0}> Conv {1}: id {2}</color>", GetActorColor(subtitle),
        //     DialogueManager.CurrentConversationState.subtitle.dialogueEntry.conversationID,
        //     DialogueManager.CurrentConversationState.subtitle.dialogueEntry.id));

        if (subtitle == null | subtitle.formattedText == null | string.IsNullOrEmpty(subtitle.formattedText.text)) return;
        string speakerName = (subtitle.speakerInfo != null && subtitle.speakerInfo.transform != null) ? subtitle.speakerInfo.transform.name : "(null speaker)";
        string temp = (string.Format("<color={0}>{1}: {2} {3}: {4}</color>",
            GetActorColor(subtitle),
            //                DialogueManager.CurrentConversationState.subtitle.dialogueEntry.conversationID,
            //                DialogueManager.CurrentConversationState.subtitle.dialogueEntry.id,
            subtitle.dialogueEntry.conversationID,
            subtitle.dialogueEntry.id,
            speakerName,
            subtitle.formattedText.text));
        EchoMessage(temp);
    }

    public void OnConversationEnd(Transform actor)
    {
        string temp = (string.Format("{0}: Ending conversation with {1}", name, GetActorName(actor)));
        EchoMessage(temp);
    }

    private string GetActorName(Transform actor)
    {
        return (actor != null) ? actor.name : "(null transform)";
    }

    private string GetActorColor(Subtitle subtitle)
    {
        if (subtitle == null | subtitle.speakerInfo == null) return "white";
        return Tools.ToWebColor(subtitle.speakerInfo.IsPlayer ? playerColor : npcColor);
    }

}


