﻿using UnityEngine;
using System;
using System.Collections;
using UnityEngine.UI;
using System.IO;
using System.Collections.Generic;
 


/// <summary>
/// This file is about file I/O with the file: "arFileName" which is stored as a text file.
/// It is suppose to be a log file, the contents of which come from ARConversationLogger.cs
/// 
/// Start: initializes the database
/// ARSaveToFile: triggers the saving of data to the file
/// 
/// </summary>
public class FileArchive : MonoBehaviour
{
    [SerializeField]
    string arFullDirectory;
    public string arVersionId;
    public string arFileName;
    [SerializeField]
    string arFullPath;
    bool initialized = false;
//    public bool archiveToFile = true;

    List<string> myData = new List<string>() { };


    void Start()
    {
        initialized = false;
        DontDestroyOnLoad(gameObject);   //make the gameObject persistant
        InitializeFileLocation();        //Get the properpath
        InitializeFile();
        initialized = true;     //don't use this until it's been initialized
    }

    private void InitializeFile()
    {
        myData.Add("CEEG Game Log File");
        myData.Add("Version ID: " + arVersionId);
        myData.Add("Unique ID: " +System.Guid.NewGuid().ToString());
        myData.Add("Created on: " + DateTime.Now.ToString("yyyy-MM-dd hh mmm"));
        myData.Add("---------------------------------------------------------");
        ARSaveToFile();
    }

    private void InitializeFileLocation()
    {
        string temp = (arVersionId == null)? Application.persistentDataPath: Path.Combine(Application.persistentDataPath, arVersionId);
        SetDirectory(temp);
        SetFileName("CEEG notes " + DateTime.Now.ToString("yyyy-MM-dd hh mmm") + ".txt");
        arFullPath = Path.Combine(arFullDirectory, arFileName);
//        arFullPath = SetFullPath(arFullPath);   //verify its ok.
    }

    public string GetFullPathAsString() {
        if(initialized) return arFullPath;
        return gameObject.name + " path not initialized";
    }
    public string SetFullPath(string s) {
        //if the user is going to change the path and file name, this routine checks it and returns a valid path.
        string arFullDirectory = CheckDirectoryExists(Path.GetDirectoryName(s));
        string arFileName = Path.GetFileName(s);
        string arFullPath = Path.Combine(arFullDirectory, arFileName);
        return arFullPath;
    }
    public void SetDirectory(string s)
    {
            if (s != null) arFullDirectory = CheckDirectoryExists(s);  //if the directory is valid, set it.  Otherwise create it, if that doesn't work use persistant datapath.
    }
    public void SetFileName(string s)
    {
        if(s != null)  arFileName = s;
    }

    public void ARSaveToFile() {
        ARSaveToFile(myData, arFullPath);
    }
    public void ARSaveToFile( List<string> data) {
        arFullPath = Path.Combine(CheckDirectoryExists(arFullDirectory), arFileName);
        ARSaveToFile(data,arFullPath);
    }
    public void ARSaveToFile(List<string> data, string aFullPath) {
        Debug.Log("Saving data to file..."+aFullPath);
        WriteListToFile(aFullPath, data);

    }

    string CheckDirectoryExists(string directoryPath)
    {   //if the directory is valid, set it.  Otherwise create it, if that doesn't work use persistant datapath.
        string p = "";
        try
        {
            p = Path.GetDirectoryName(directoryPath);
            // If the directory doesn't exist, create it.
            if (!Directory.Exists(p))
            {
                Directory.CreateDirectory(p);
            }
        }
        catch (System.Exception e)
        {
            p = Application.persistentDataPath;
            print(e);
        }
        finally{ }
        return p;
    }

    void WriteListToFile(string location, List<string> data) {
        //using (StreamWriter outputFile = new StreamWriter(mydocpath + @"\WriteLines.txt"))
        try
        {
            using (FileStream fs = new FileStream(location, FileMode.Append, FileAccess.Write))
            {
                using (StreamWriter sw = new StreamWriter(fs))
                {
                    foreach (string line in data)
                        sw.WriteLine(line);
                }
            }
            Debug.Log("Save Complete..."+location);
        }
        catch (System.Exception e)
        {
            print("ERROR --- FileArchive.cs writing to file failed");
            print(e.ToString());
        }
    }

    public List<string> ARReadFromFile(string location)
    {//http://answers.unity3d.com/questions/722539/return-a-list-from-a-method-in-c.html
        Debug.Log("Reading from file...");
        myData.Clear();
        try
        {
            using (StreamReader dataFile = new StreamReader(location))
            {
                string line;
                int counter=0;
                while (!dataFile.EndOfStream)               //exit when you reach the end of the file.
                {
                    if ((line = dataFile.ReadLine()) != null)     //skip null entries. 
                    {
                        myData.Add(line);
                        counter++;
                    }
                }
                print(counter + " Lines read");
            }
        }
        catch (System.Exception e)
        {
            print("The file could not be read:");
            print(e.Message);
        }
        finally {  }
        return myData;
    }

    public void AddTextToFile(string text) {
        List<string> temp = new List<string>();
        temp.Add(text);
        AddTextToFile(temp);
    }
    public void AddTextToFile(List<string> text) {
        myData.AddRange(text);
        WriteListToFile(arFullPath,text); //Streamwriter adds the end of the file by default, this "list" should just get added!
    }

    public List<string> GetDataAsList() {
        return myData;
    }
    public string GetDataAsString() { 
        string newText = "";
        foreach (string s in myData)
        {
            newText = newText + s + "\n";
        }
        return newText;
    }
    public void SaveStringToFile(string data) {
        try
        {
            using (FileStream fs = new FileStream(arFullPath, FileMode.Create, FileAccess.Write))
            {
                using (StreamWriter sw = new StreamWriter(fs))
                {
                        sw.WriteLine(data);
                }
            }
            Debug.Log("Save Complete..." + arFullPath);
        }
        catch (System.Exception e)
        {
        print("ERROR writing to file " + arFullPath);
            print(e.ToString());
        }
    }
}